<?php
ini_set('display_errors', 1);
 error_reporting(E_ALL);
$nzshpcrt_gateways[$num] = array(
	'name'                   => __( 'Authorize.net', 'wpsc' ),
	'api_version'            => 2.0,
	'class_name'             => 'wpsc_merchant_authorize',
	'has_recurring_billing'  => true,
	'wp_admin_cannot_cancel' => true,
	'display_name'			 => __( 'Authorize.net', 'wpsc' ),
	'image' => WPSC_URL . '/images/cc.gif',
	'requirements'           => array(
		'php_version'        => 4.3,    // so that you can restrict merchant modules to PHP 5, if you use PHP 5 features
		'extra_modules'      => array() // for modules that may not be present, like curl
	),
	'form'                   => 'form_authorize',
	'submit_function'        => 'submit_authorize',
	'internalname'           => 'wpsc_merchant_authorize', // this may be legacy, not yet decided
	// All array members below here are legacy, and use the code in authorize_multiple.php
	//	'form' => 'form_authorize_multiple',
	//	'submit_function' => 'submit_authorize_multiple',
	'payment_type'           => 'authorize',
	'supported_currencies'   => array(
		'currency_list'      => array( 'AUD', 'BRL', 'CAD', 'CHF', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD', 'HUF', 'ILS', 'JPY', 'MXN', 'MYR', 'NOK', 'NZD', 'PHP', 'PLN', 'SEK', 'SGD', 'THB', 'TWD', 'USD' ),
		'option_name'        => 'authorize_curcode'
	)
);

/**
 * WP eCommerce Authorize.net Standard Merchant Class
 *
 * This is the authorize standard merchant class, it extends the base merchant class
 *
 * @package wp-e-commerce
 * @since 3.7.6
 * @subpackage wpsc-merchants
 */
class wpsc_merchant_authorize extends wpsc_merchant {

	var $name              = '';
	var $authorize_ipn_values = array( );

	function __construct( $purchase_id = null, $is_receiving = false ) {
		$this->name = __( 'Authorize.net', 'wpsc' );
		parent::__construct( $purchase_id, $is_receiving );
	}

	function get_local_currency_code() {
		if ( empty( $this->local_currency_code ) ) {
			global $wpdb;
			$this->local_currency_code = $wpdb->get_var( $wpdb->prepare( "SELECT `code` FROM `".WPSC_TABLE_CURRENCY_LIST."` WHERE `id`= %d LIMIT 1", get_option( 'currency_type' ) ) );
		}

		return $this->local_currency_code;
	}

	function get_authorize_currency_code() {
		if ( empty( $this->authorize_currency_code ) ) {
			global $wpsc_gateways;
			$this->authorize_currency_code = $this->get_local_currency_code();

			if ( ! in_array( $this->authorize_currency_code, $wpsc_gateways['wpsc_merchant_authorize']['supported_currencies']['currency_list'] ) )
				$this->authorize_currency_code = get_option( 'authorize_curcode', 'USD' );
		}

		return $this->authorize_currency_code;
	}

	/**
	 * construct value array method, converts the data gathered by the base class code to something acceptable to the gateway
	 * @access public
	 */
	function construct_value_array() {
		
		// Store settings to be sent to authorize
	
		$data = array( );
		$data['LOGINID']      = get_option( 'authorize_loginid' );
		$data['TXNKEY']       = get_option( 'authorize_txn_key' );
		
		$data['CURRENCYCODE'] = $this->get_authorize_currency_code();

		// Basic Cart Data
		$data['INVNUM']          = $this->cart_data['session_id'];
		$data['IPADDRESS']       = $_SERVER["REMOTE_ADDR"];
		$data['METHOD']       = 'CC';

		if ( $this->cart_data['billing_address']['country'] == 'UK' )
			$this->cart_data['billing_address']['country'] = 'GB';

		// Billing Data
		$data['FIRSTNAME']   = $this->cart_data['billing_address']['first_name'];
		$data['LASTNAME']    = $this->cart_data['billing_address']['last_name'];
		$data['EMAIL']       = $this->cart_data['email_address'];
		$data['STREET']      = $this->cart_data['billing_address']['address'];
		$data['CITY']        = $this->cart_data['billing_address']['city'];
		$data['STATE']       = $this->cart_data['billing_address']['state'];
		$data['COUNTRYCODE'] = $this->cart_data['billing_address']['country'];
		$data['ZIP']         = $this->cart_data['billing_address']['post_code'];

		// Shipping Data
		$data['SHIPTONAME']    = $this->cart_data['shipping_address']['first_name'] . " " . $this->cart_data['shipping_address']['last_name'];
		$data['SHIPTOSTREET']  = $this->cart_data['shipping_address']['address'];
		$data['SHIPTOCITY']    = $this->cart_data['shipping_address']['city'];

		// Check the state for internal numeric ID and trap it
		if ( is_numeric( $this->cart_data['shipping_address']['state'] ) )
			$this->cart_data['shipping_address']['state'] = wpsc_get_state_by_id( $this->cart_data['shipping_address']['state'], 'code' );

		if ( $this->cart_data['shipping_address']['country'] == 'UK' )
			$this->cart_data['shipping_address']['country'] = 'GB';

		$data['SHIPTOSTATE']   = $this->cart_data['shipping_address']['state'];
		$data['SHIPTOCOUNTRY'] = $this->cart_data['shipping_address']['country'];
		$data['SHIPTOZIP']     = $this->cart_data['shipping_address']['post_code'];

		// Credit Card Data
		$data['CREDITCARDTYPE'] = $_POST['cctype'];
		$data['ACCT']           = str_replace( array(' ', '-'), '', $_POST['card_number'] );
		$data['EXPDATE']        = $_POST['expiry']['month'] . $_POST['expiry']['year'];
		$data['CVV2']           = $_POST['card_code'];

		// Ordered Items

		// Cart Item Data
		$i = $item_total = 0;
		$tax_total = wpsc_tax_isincluded() ? 0 : $this->cart_data['cart_tax'];

		$shipping_total = $this->convert( $this->cart_data['base_shipping'] );

		foreach ( $this->cart_items as $cart_row ) {
			$data['L_NAME' . $i] = apply_filters( 'the_title', $cart_row['name'] );
			$data['L_AMT' . $i] = $this->convert( $cart_row['price'] );
			$data['L_NUMBER' . $i] = $i;
			$data['L_QTY' . $i] = $cart_row['quantity'];

			$shipping_total += $this->convert( $cart_row['shipping'] );
			$item_total += $this->convert( $cart_row['price'] ) * $cart_row['quantity'];

			$i++;
		}

		if ( $this->cart_data['has_discounts'] ) {
			$discount_value = $this->convert( $this->cart_data['cart_discount_value'] );

			$coupon = new wpsc_coupons( $this->cart_data['cart_discount_data'] );

			// free shipping
			if ( $coupon->is_percentage == 2 ) {
				$shipping_total = 0;
				$discount_value = 0;
			} elseif ( $discount_value >= $item_total ) {
				$discount_value = $item_total - 0.01;
				$shipping_total -= 0.01;
			}

			$data["L_NAME{$i}"] = _x( 'Coupon / Discount', 'Authorize.net Item Name for Discounts', 'wpsc' );
			$data["L_AMT{$i}"] = - $discount_value;
			$data["L_NUMBER{$i}"] = $i;
			$data["L_QTY{$i}"] = 1;
			$item_total -= $discount_value;
		}

		// Cart totals
		$data['ITEMAMT'] = $this->format_price( $item_total );
		$data['SHIPPINGAMT'] = $this->format_price( $shipping_total );
		$data['TAXAMT'] = $this->convert( $tax_total );
		$data['AMT'] = $data['ITEMAMT'] + $data['SHIPPINGAMT'] + $data['TAXAMT'];
		
		return $data;
	}

	/**
	 * submit method, sends the received data to the payment gateway
	 * @access public
	 */
	function submit() {
		
			require_once(WPSC_FILE_PATH.'/wpsc-merchants/library/AuthnetAIM.class.php');
			
			$auth_data=$this->construct_value_array();					
			
			if(get_option( 'authorize_testmode' ) == "on")
			$payment = new AuthnetAIM($auth_data['LOGINID'], $auth_data['TXNKEY'],true); //true for test mode
			else
			$payment = new AuthnetAIM($auth_data['LOGINID'], $auth_data['TXNKEY']); //true for test mode
			
		
			$payment->setParameter("x_amount", $auth_data['AMT']);	
			$payment->setTransaction($auth_data['ACCT'], $auth_data['EXPDATE'], $auth_data['AMT'], $auth_data['CVV2']);
			
			
			$payment->setParameter("x_first_name", $auth_data['FIRSTNAME']);
			
			if(!empty($auth_data['LASTNAME']))
		    $payment->setParameter("x_last_name", $auth_data['LASTNAME']);
			
			if(!empty($auth_data['LASTNAME']))
			$payment->setParameter("x_address", $auth_data['LASTNAME']);
			
			if(!empty($auth_data['CITY']))
			$payment->setParameter("x_city", $auth_data['CITY']);
			
			if(!empty($auth_data['STATE']))
			$payment->setParameter("x_state", $auth_data['STATE']);
			
			if(!empty($auth_data['ZIP']))
			$payment->setParameter("x_zip", $auth_data['ZIP']);
			
			$payment->setParameter("x_description", 'Purple Daisy Checkout');
		
			if(!empty($auth_data['METHOD']))
				$payment->setParameter("x_method", $auth_data['METHOD']);
		
			
			$payment->process();
			
			
			if($payment->isApproved())
			{
				
				$x_auth_code = $payment->getAuthCode();							
				
			    $this->set_transaction_details( $payment->getTransactionID(), 3 );
				$this->go_to_transaction_results( $this->cart_data['session_id'] );		
				
				
				
			} 
			else 
			{
				$response_code = $payment->getResponseCode(); 
				$reason = $payment->getResponseText(); 
				$this->set_error_message( $reason );
			}
	
	}

	function format_price( $price ) {
		$authorize_currency_code = get_option( 'authorize_curcode' );

		switch ( $authorize_currency_code ) {
			case "JPY":
				$decimal_places = 0;
				break;

			case "HUF":
				$decimal_places = 0;

			default:
				$decimal_places = 2;
				break;
		}

		$price = number_format( sprintf( "%01.2f", $price ), $decimal_places, '.', '' );

		return $price;
	}

	function convert( $amt ){
		if ( empty( $this->rate ) ) {
			$this->rate = 1;
			$authorize_currency_code = $this->get_authorize_currency_code();
			$local_currency_code = $this->get_local_currency_code();
			if( $local_currency_code != $authorize_currency_code ) {
				$curr=new CURRENCYCONVERTER();
				$this->rate = $curr->convert( 1, $authorize_currency_code, $local_currency_code );
			}
		}

		return $this->format_price( $amt * $this->rate );
	}

}

function submit_authorize() {
	if ( isset( $_POST['Authorize']['authorize_loginid'] ) )
		update_option( 'authorize_loginid', $_POST['Authorize']['authorize_loginid'] );

	if ( isset( $_POST['Authorize']['authorize_txn_key'] ) )
		update_option( 'authorize_txn_key', $_POST['Authorize']['authorize_txn_key'] );

	if(isset($_POST['authorize_curcode']))
		update_option('authorize_curcode', $_POST['authorize_curcode']);
		
	if ( isset( $_POST['Authorize']['testmode'] ) )
		update_option( 'authorize_testmode', $_POST['Authorize']['testmode'] );

	return true;
}

function form_authorize() {
	global $wpsc_gateways, $wpdb;
	if ( get_option( 'authorize_testmode' ) == "on" )
		$selected = 'checked="checked"';
	else
		$selected = '';

	$output = '
	<tr>
		<td>
			<label for="authorize_username">' . __( 'Login ID:', 'wpsc' ) . '</label>
		</td>
		<td>
			<input type="text" name="Authorize[authorize_loginid]" id="authorize_loginid" value="' . get_option( "authorize_loginid" ) . '" size="30" />
		</td>
	</tr>
	<tr>
		<td>
			<label for="authorize_password">' . __( 'Transaction Key:', 'wpsc' ) . '</label>
		</td>
		<td>
			<input type="password" name="Authorize[authorize_txn_key]" id="authorize_txn_key" value="' . get_option( 'authorize_txn_key' ) . '" size="16" />
		</td>
	</tr>
	
	<tr>
		<td>
			<label for="authorize_testmode">' . __( 'Test Mode Enabled:', 'wpsc' ) . '</label>
		</td>
		<td>
			<input type="hidden" name="Authorize[testmode]" value="off" /><input type="checkbox" name="Authorize[testmode]" id="authorize_testmode" value="on" ' . $selected . ' />
		</td>
	</tr>
';

	$store_currency_code = $wpdb->get_var( $wpdb->prepare( "SELECT `code` FROM `".WPSC_TABLE_CURRENCY_LIST."` WHERE `id` IN (%d)", get_option( 'currency_type' ) ) );
	$current_currency = get_option('authorize_curcode');

	if(($current_currency == '') && in_array($store_currency_code, $wpsc_gateways['wpsc_merchant_authorize']['supported_currencies']['currency_list'])) {
		update_option('authorize_curcode', $store_currency_code);
		$current_currency = $store_currency_code;
	}
	if($current_currency != $store_currency_code) {
		$output .= "<tr> <td colspan='2'><strong class='form_group'>" . __( 'Currency Converter', 'wpsc' ) . "</td> </tr>
		<tr>
			<td colspan='2'>".__('Your website is using a currency not accepted by Authorize.net, select an accepted currency using the drop down menu below. Buyers on your site will still pay in your local currency however we will convert the currency and send the order through to Authorize.net using the currency you choose below.', 'wpsc')."</td>
		</tr>\n";

		$output .= "<tr>\n <td>" . __('Convert to', 'wpsc' ) . " </td>\n ";
		$output .= "<td>\n <select name='authorize_curcode'>\n";

		if (!isset($wpsc_gateways['wpsc_merchant_authorize']['supported_currencies']['currency_list']))
			$wpsc_gateways['wpsc_merchant_authorize']['supported_currencies']['currency_list'] = array();

		$authorize_currency_list = array_map( 'esc_sql', $wpsc_gateways['wpsc_merchant_authorize']['supported_currencies']['currency_list'] );

		$currency_list = $wpdb->get_results("SELECT DISTINCT `code`, `currency` FROM `".WPSC_TABLE_CURRENCY_LIST."` WHERE `code` IN ('".implode("','",$authorize_currency_list)."')", ARRAY_A);
		foreach($currency_list as $currency_item) {
			$selected_currency = '';
			if($current_currency == $currency_item['code']) {
				$selected_currency = "selected='selected'";
			}
			$output .= "<option ".$selected_currency." value='{$currency_item['code']}'>{$currency_item['currency']}</option>";
		}
		$output .= "            </select> \n";
		$output .= "          </td>\n";
		$output .= "       </tr>\n";
	}
	return $output;
}

$years = $months = '';

if ( in_array( 'wpsc_merchant_authorize', (array)get_option( 'custom_gateway_options' ) ) ) {

	$curryear = date( 'Y' );

	//generate year options
	for ( $i = 0; $i < 10; $i++ ) {
		$years .= "<option value='" . $curryear . "'>" . $curryear . "</option>\r\n";
		$curryear++;
	}

	$output = "
	<tr>
		<td class='wpsc_CC_details'>" . __( 'Credit Card Number *', 'wpsc' ) . "</td>
		<td>
			<input type='text' value='' name='card_number' />
		</td>
	</tr>
	<tr>
		<td class='wpsc_CC_details'>" . __( 'Credit Card Expiry *', 'wpsc' ) . "</td>
		<td>
			<select class='wpsc_ccBox' name='expiry[month]'>
			" . $months . "
			<option value='01'>01</option>
			<option value='02'>02</option>
			<option value='03'>03</option>
			<option value='04'>04</option>
			<option value='05'>05</option>
			<option value='06'>06</option>
			<option value='07'>07</option>
			<option value='08'>08</option>
			<option value='09'>09</option>
			<option value='10'>10</option>
			<option value='11'>11</option>
			<option value='12'>12</option>
			</select>
			<select class='wpsc_ccBox' name='expiry[year]'>
			" . $years . "
			</select>
		</td>
	</tr>
	<tr>
		<td class='wpsc_CC_details'>" . __( 'CVV *', 'wpsc' ) . "</td>
		<td><input type='text' size='4' value='' maxlength='4' name='card_code' />
		</td>
	</tr>
	<tr>
		<td class='wpsc_CC_details'>" . __( 'Card Type *', 'wpsc' ) . "</td>
		<td>
		<select class='wpsc_ccBox' name='cctype'>";

	$card_types = array(
		'Visa' => __( 'Visa', 'wpsc' ),
		'Mastercard' => __( 'MasterCard', 'wpsc' ),
		'Discover' => __( 'Discover', 'wpsc' ),
		'Amex' => __( 'Amex', 'wpsc' ),
	);
	$card_types = apply_filters( 'wpsc_authorize_accepted_card_types', $card_types );
	foreach ( $card_types as $type => $title ) {
		$output .= sprintf( '<option value="%1$s">%2$s</option>', $type, esc_html( $title ) );
	}
	$output .= "</select>
		</td>
	</tr>
";

$gateway_checkout_form_fields[$nzshpcrt_gateways[$num]['internalname']] = $output;

}
?>
